/*
* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* (C) Copyright IBM Corp. 2012, 2013
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/

/*!\file
 *\verbatim
 *--------------------------------------------------------------------*
 * File:    exejar.c
 * Desc:     Generic executable to run Java jar file for command-line utilities,
 * 			  such as MoDApp, etc.
 *
 * Function: This file contains the following functions:
 *
 *       main()
 *
 * Author:   Peter Hagelund
 * Created:  9/10/12
 *
 *
 *--------------------------------------------------------------------*
\endverbatim */

#include <stdio.h>
#include <string.h>
#include <jni.h>
#include <ctype.h>

static const char *classPathOption = "-Djava.class.path=";
static const char *exeSuffix = ".exe";
static const char *jarSuffix = ".jar";
static const char *classBasePath = "com/ibm/nex/";
static const char *initialClassName = "Main";

#if defined(WIN32)
static const char pathSepChar = '\\';
#else
static const char pathSepChar = '/';
#endif

static char *getBasename(char *);

#if !defined(strrchr)
char * strrchr(const char *inStr, int c)
{
	char *retPtr = (char *)NULL;
	char *cPtr = inStr;
	int cVal;

	if (inStr == (char *)NULL) {
		return ((char *)NULL);
	}

	while ((cVal = *cPtr++) != '\0')
	{
		if (cVal == c) {
			retPtr = cPtr - 1; // back the pointer down, since we incremented
		}
	}
	return (retPtr);
}
#endif

/*!\verbatim
 *--------------------------------------------------------------------*
 * Func:     main
 *
 * Purpose:  Takes the executable name from argv[0], removes any suffix, and locates the
 *           jar file named {executable_name}.jar.
 *           functions to acquire an OCI Statement handle and ROWID descriptor
 *
 * Parms:
 *     Passes command line parameters to Java class for execution.
 *
 * Returns:
 *		Return code from Java method invoked.
 *
 *--------------------------------------------------------------------*
\endverbatim */

int main(int argc, const char * argv[])
{
    JavaVMOption options[1];
    char optionString[1024];
    char programName[1024]; // Overkill, but consistent with optionString
    char *baseProgramName;
    char className[1024]; // Again overkill.
    JavaVM *javaVM;
    JNIEnv *env;
    size_t offset;
    JavaVMInitArgs initArgs;
    jint rc;
    jclass loaderClass;
    jmethodID constructorMID;
    jmethodID executeMID;
    jclass stringClass;
    jsize length = argc - 1;
    jobjectArray args;
    int i;
    jobject loader;

    (void)strcpy(optionString, classPathOption);
    (void) strcat(optionString, argv[0]);
    (void)strcpy(programName,argv[0]);
    offset = strlen(optionString) - 4;
    // If we're on Windows with a .exe, simply replace '.exe' with '.jar' - otherwise append .jar
    if (strcmp(optionString + offset, exeSuffix) == 0) {
        (void)strcpy(optionString + offset, jarSuffix);
        *(programName+offset) = '\0'; // Truncate .exe to get program name
    } else {
        (void)strcat(optionString, jarSuffix);
    }
    baseProgramName = getBasename(programName);
    options[0].optionString = optionString;

    initArgs.version = JNI_VERSION_1_6;
    initArgs.nOptions = 1;
    initArgs.options = options;
    initArgs.ignoreUnrecognized = JNI_TRUE;
    rc = JNI_CreateJavaVM(&javaVM, (void**)&env, &initArgs);
    if (rc < 0) {
        (void)fprintf(stderr, "Error: Unable to create JVM.\n");
        return 12;
    }
    /* We have established a convention -- the class name is com.ibm.nex.{programName}.Main
     * The class can contain a static main() method that instantiates the Main class and
     * invokes execute()
     */
    (void)strcpy(className,classBasePath);
    (void)strcat(className,baseProgramName);
    (void)strcat(className,"/");
    (void)strcat(className,initialClassName);

    loaderClass = (*env)->FindClass(env, className);
    if (loaderClass == NULL) {
        (void)fprintf(stderr, "Error: Unable to find loader class. %s\n", className);
        return 12;
    }
    constructorMID = (*env)->GetMethodID(env, loaderClass, "<init>", "([Ljava/lang/String;)V");
    if (constructorMID == NULL) {
        (void)fprintf(stderr, "Error: Unable to find loader constructor.");
        return 12;
    }
    executeMID = (*env)->GetMethodID(env, loaderClass, "execute", "()I");
    if (executeMID == NULL) {
        (void)fprintf(stderr, "Error: Unable to find loader execute method.");
        return 12;
    }
    stringClass = (*env)->FindClass(env, "java/lang/String");

    args = (*env)->NewObjectArray(env, length, stringClass, NULL);
    for (i = 0; i < length; i++) {
        jstring arg = (*env)->NewStringUTF(env, argv[i + 1]);
        (*env)->SetObjectArrayElement(env, args, i, arg);
        (*env)->DeleteLocalRef(env, arg);
    }
    loader = (*env)->NewObject(env, loaderClass, constructorMID, args);
    if (loader == NULL) {
        (void)fprintf(stderr, "Error: Unable to instantiate loader.\n");
        return 12;
    }
    (*env)->DeleteLocalRef(env, args);
    rc = (*env)->CallIntMethod(env, loader, executeMID);
    (*env)->DeleteLocalRef(env, loader);
    (*env)->DeleteLocalRef(env, stringClass);
    (*env)->DeleteLocalRef(env, loaderClass);
    (*javaVM)->DestroyJavaVM(javaVM);
    return rc;
}

char *getBasename(char *progStr)
{
	char *ptr = strrchr(progStr,(int)pathSepChar);
	if (ptr == (char *)NULL) {
		return (progStr);
	}
	return (++ptr);
}




